#include "CommunicationProtocol.h"

#define DEBUGGING 0
#define COORDS 48
#define PIN_30V   A0 // PC0 ADC0
#define PIN_TRAFO A1  
#define PIN_X     A2  
#define PIN_Y     A3  
#define PIN_INT0  2    
#define FACTOR_T  867
#define FACTOR_N  47

struct f {
    //uint16_t min_coord, max_coord; // max scale value in mV (positive or negative)
    uint16_t Vx, Vy;                 // max scale value in mV (positive or negative)
    uint16_t coords = COORDS;        // number of coordinates stored in x,y
    int16_t x[COORDS], y[COORDS];    // sample data    
};

volatile bool startpulse;
unsigned char buffer[sizeof( struct f )];
class communication_package *cp_out = new class communication_package;
struct f fields;
bool bOutputReady = false;
int32_t base;

/////////////////////////////////////////////////////////////////////////////////////////////
void int0() {
    startpulse = true;
}
/////////////////////////////////////////////////////////////////////////////////////////////
unsigned long us_diff( void ) { // returns the elapsed microseconds since the last call
    static unsigned long us_old = 0;
    unsigned long retval, us_new = micros();   // micros() will overflow after 67 seconds
    if( us_new < us_old ) {                    // this function can correct a single overflow
        retval = ( ( us_new + 0x4000000 ) - us_old );
    } else {
        retval = ( us_new - us_old );
    }
    us_old = us_new;
    return( retval );
}
/////////////////////////////////////////////////////////////////////////////////////////////
void setup() {
    Serial.begin( 115200 );  
    attachInterrupt(digitalPinToInterrupt(PIN_INT0), int0, FALLING );
}
/////////////////////////////////////////////////////////////////////////////////////////////
long convert( int value, int maxval ){
    return ( value-base ) * FACTOR_T * 3460 / 1023 / FACTOR_N; // measured VCC 3.46V
}
/////////////////////////////////////////////////////////////////////////////////////////////
void loop() {
    static uint32_t us, us_counter = 0;
    int16_t arr_x[48], arr_y[48], arr_t[48];
    us_counter += us_diff();    
    if( us_counter >= 20000 ) { // every 20ms
        us_counter -= 20000;
        fields.Vx = 10000; // max scale horizontal            
        fields.Vy = 10000; // max scale vertical         
        base =  analogRead( PIN_30V );
        startpulse = false;
        while ( startpulse == false && ! DEBUGGING );
        us = 420 + micros();        
        if ( us < 0xFFFFFFFF - 420 * 48 ) {
            for ( int i = 0; i < 48; i++) {
                arr_t[i]  = analogRead( PIN_TRAFO );
                fields.x[i]  = analogRead( PIN_X );
                fields.y[i]  = analogRead( PIN_Y );
                while ( micros() < us ); // wacht 420 us
                //digitalWrite(10, HIGH);
                //digitalWrite(10, LOW);
                us += 420;
            } 
            for ( int i = 0; i < 48; i++) {
                fields.x[i] = fields.Vx + convert( fields.x[i], fields.Vx );
                fields.y[i] = fields.Vy + convert( fields.y[i], fields.Vy );                         
            }
            ///  DEBUG  ///
            /*
            if( DEBUGGING ) {
                for ( int i = 0; i < 48; i++) {  
                    fields.x[i] = 2048;
                    fields.y[i] = 2048;
                }
                fields.x[0] = convert( 100, fields.Vx );
                fields.y[0] = convert( 100, fields.Vy );       
                fields.x[47] = convert( 4096-100, fields.Vx );
                fields.y[47] = convert( 4096-100, fields.Vy ); 
            }
            */
            ///  DEBUG  ///
                        
            bOutputReady = true;
        }
    }
    if( bOutputReady ) {
        bOutputReady = false;
        memcpy( &buffer, &fields, sizeof( struct f ) );
        cp_out->pkg_clear(); 
        for( int i = 0; i < sizeof( buffer ); i++ ) {
            if( cp_out->pkg_add_byte( buffer[i] ) ) {
                break;
            }
        }
        cp_out->pkg_send();   
    }
}
